import { TypedEmitter } from 'tiny-typed-emitter';
import { CrosslabConnectionTransport } from './deviceConnectionTransport.js';
import { isClosePeerConnectionMessage, isCommandMessage, isConfigurationMessage, isCreatePeerConnectionMessage, isExperimentStatusChangedMessage, isSignalingMessage } from './deviceMessages.js';
import { crosslabTransport, logger } from './logging.js';
import { WebRTCPeerConnection } from './peer/webrtc-connection.js';
export class DeviceHandler extends TypedEmitter {
    evaluateState(input, output) {
        const some = input.some(i => this.stateEvaluator?.(i, output));
        const every = input.every(i => this.stateEvaluator?.(i, output));
        if (some) {
            console.log("Error Detected!");
            if (every) {
                console.log("User Error!");
            }
        }
        else {
            console.log("No Error Detected!");
        }
    }
    handleDataUpstream(data) {
        //let state = Object.values<ServiceT>(this.services).map(service => service.get_state());
        let state = {};
        for (const serviceId in this.services) {
            state[serviceId] = this.services[serviceId].get_state();
        }
        let fullstate = {};
        for (const fstate of this.fullstatemap.values()) {
            for (const id in fstate) {
                fullstate[id] = [...fullstate[id] ?? [], ...fstate[id]];
            }
        }
        this.evaluateState(fullstate[this.id] ?? [], state);
        fullstate[this.id] = [state];
        return { fullstate: { [this.id]: fullstate }, data };
    }
    ;
    handleDataDownstream(data) {
        if ("fullstate" in data) {
            for (const device in data["fullstate"]) {
                this.fullstatemap.set(device, data["fullstate"][device]);
            }
            return data["data"];
        }
        return data;
    }
    ;
    constructor(...services) {
        super();
        this.connections = new Map();
        this.services = {};
        this.connectionConstuctors = new Map;
        this.fullstatemap = new Map;
        this.id = crypto.randomUUID();
        for (const service of services) {
            this.addService(service);
        }
        this.connectionConstuctors.set("", () => (new WebRTCPeerConnection({
            iceServers: [
                { urls: 'stun:stun.goldi-labs.de:3478' },
                { urls: 'turn:turn.goldi-labs.de:3478', username: 'goldi', credential: 'goldi' },
            ],
        })));
    }
    setErrorState(stateEvaluator) {
        this.stateEvaluator = stateEvaluator;
    }
    async connect(transportOrconnectOptions) {
        let transport;
        if ('connect' in transportOrconnectOptions) {
            transport = transportOrconnectOptions;
        }
        else {
            transport = new CrosslabConnectionTransport(transportOrconnectOptions);
        }
        return this._connect(transport);
    }
    async _connect(transport) {
        this.transport = transport;
        this.transport.connect(crosslabTransport);
        this.transport.on("message", message => {
            if (isCommandMessage(message)) {
                if (isCreatePeerConnectionMessage(message)) {
                    this.handleCreatePeerConnectionMessage(message);
                }
                else if (isClosePeerConnectionMessage(message)) {
                    this.handleClosePeerConnectionMessage(message);
                }
            }
            if (isSignalingMessage(message)) {
                this.handleSignalingMessage(message);
            }
            if (isConfigurationMessage(message)) {
                this.handleConfigurationMessage(message);
            }
            if (isExperimentStatusChangedMessage(message)) {
                this.handleExperimentStatusChangedMessage(message);
            }
        });
    }
    addService(service) {
        this.services[service.serviceId] = service;
    }
    handleCreatePeerConnectionMessage(message) {
        if (this.connections.has(message.connectionUrl)) {
            throw Error('Can not create a connection. Connection Id is already present');
        }
        logger.log('info', 'creating connection', message);
        const connectionConstuctor = this.connectionConstuctors.get(message.connectiontype);
        if (connectionConstuctor === undefined) {
            throw Error("No Connection Constructor for " + message.connectiontype + " available.");
        }
        const connection = connectionConstuctor(message);
        connection.tiebreaker = message.tiebreaker;
        connection.channelExtension = { upstream: (data) => this.handleDataUpstream(data), downstream: (data) => this.handleDataDownstream(data) };
        this.connections.set(message.connectionUrl, connection);
        for (const serviceConfig of message.services) {
            const service = this.services[serviceConfig.serviceId];
            if (service === undefined) {
                throw Error('No Service for the service config was found');
            }
            service.setupConnection(connection, serviceConfig);
        }
        connection.on('signalingMessage', msg => {
            logger.log('info', 'sending:', msg);
            this.transport?.send(JSON.stringify({
                ...msg,
                messageType: 'signaling',
                connectionUrl: message.connectionUrl,
            }));
        });
        connection.on('connectionChanged', () => {
            const connectionStateChangedMessage = {
                messageType: 'connection-state-changed',
                status: connection.state,
                connectionUrl: message.connectionUrl,
            };
            this.transport?.send(JSON.stringify(connectionStateChangedMessage));
            this.emit('connectionsChanged');
        });
        connection.connect();
        this.emit('connectionsChanged');
    }
    handleSignalingMessage(message) {
        const connection = this.connections.get(message.connectionUrl);
        if (connection === undefined) {
            throw Error('No Connection for the signaling message was found');
        }
        connection.handleSignalingMessage(message);
    }
    handleClosePeerConnectionMessage(message) {
        const connection = this.connections.get(message.connectionUrl);
        if (!connection) {
            throw Error('Cannot close a connection. Connection Id is not present');
        }
        logger.log('info', 'closing connection', message);
        connection.teardown();
        this.connections.delete(message.connectionUrl);
    }
    handleConfigurationMessage(message) {
        this.emit('configuration', message.configuration);
    }
    handleExperimentStatusChangedMessage(message) {
        this.emit('experimentStatusChanged', {
            status: message.status,
            message: message.message,
        });
    }
    getServiceMeta() {
        return Object.values(this.services).map(service => service.getMeta());
    }
}
//# sourceMappingURL=deviceHandler.js.map